import * as fs from 'fs';
import * as os from 'os';
import { acceptLicenseTermsAndAddDatabaseApi } from '../../../helpers/database';
import { MonitorPage } from '../../../pageObjects';
import {
    commonUrl,
    ossStandaloneConfig
} from '../../../helpers/conf';
import { rte } from '../../../helpers/constants';
import { deleteStandaloneDatabaseApi } from '../../../helpers/api/api-database';

const monitorPage = new MonitorPage();
const tempDir = os.tmpdir();

fixture `Save commands`
    .meta({ type: 'regression', rte: rte.standalone })
    .page(commonUrl)
    .beforeEach(async() => {
        await acceptLicenseTermsAndAddDatabaseApi(ossStandaloneConfig, ossStandaloneConfig.databaseName);
    })
    .afterEach(async() => {
        // Delete database
        await deleteStandaloneDatabaseApi(ossStandaloneConfig);
    });
test('Verify that when clicks on “Reset Profiler” button he brought back to Profiler home screen', async t => {
    // Start Monitor without Save logs
    await monitorPage.startMonitor();
    // Remember the number of files in Temp
    const numberOfTempFiles = fs.readdirSync(tempDir).length;
    // Reset profiler
    await monitorPage.resetProfiler();
    //Check the screen
    await t.expect(monitorPage.monitorNotStartedElement.visible).ok('The Profiler home screen not appeared');
    await t.click(monitorPage.closeMonitor);
    // Start Monitor with Save logs
    await monitorPage.startMonitorWithSaveLog();
    // Reset profiler
    await monitorPage.resetProfiler();
    // Check the screen
    await t.expect(monitorPage.monitorNotStartedElement.visible).ok('The Profiler home screen not appeared');
    await t.expect(monitorPage.monitorIsStartedText.visible).notOk('The current Profiler session is not closed');
    // temporary Log file is deleted
    await t.expect(numberOfTempFiles).eql(fs.readdirSync(tempDir).length, 'The temporary Log file is not deleted');
});
test('Verify that when user clears the Profiler he doesn\'t brought back to Profiler home screen', async t => {
    // Start Monitor
    await monitorPage.startMonitor();
    // Clear monitor and check the view
    await t.click(monitorPage.clearMonitorButton);
    await t.expect(monitorPage.monitorNotStartedElement.visible).notOk('Profiler home screen is still opened after Clear');
    await t.click(monitorPage.closeMonitor);
    // Start Monitor with Save logs
    await monitorPage.startMonitorWithSaveLog();
    // Clear monitor and check the view
    await t.click(monitorPage.clearMonitorButton);
    await t.expect(monitorPage.monitorNotStartedElement.visible).notOk('Profiler home screen is still opened after Clear');
});
